hex

vocabulary graphics immediate
graphics definitions

( Colour constants )

0 constant black
1 constant white
2 constant red
3 constant cyan
4 constant purple
5 constant green
6 constant blue
7 constant yellow

8  constant orange
9  constant lt-orange
0a constant pink
0b constant lt-cyan
0c constant lt-purple
0d constant lt-green
0e constant lt-blue
0f constant lt-yellow

( Mode flags )

1 constant double-height
2 constant bitmap
8 constant multi-colour

: border   ( col -- )
    7  and  900f  c@  0f8  and  or  900f  c!  ;

: background   ( col -- )
    4  lshift  900f  c@  0f  and  or  900f  c!  ;

: aux   ( col -- )
    4  lshift  900e  c@  0f  and  or  900e  c!  ;

: foreground   ( col -- )
    286  c!  ;

: invert   ( tf -- )
    0=  8  and  900f  c@  0f7  and  or  900f  c!  ;

( Return TRUE if NTSC, FALSE if PAL )

: ?ntsc   (  --  flag )
    0e475  C@  41  =  ;

: vic2cpu   ( vicaddr -- cpuaddr )
    dup  2000  and  if
	1fff  and  else
	8000  or  then  ;

: cpu2vic   ( cpuaddr -- vicaddr )
    dup  8000  and  if
	3fff  and  else
	2000  or  then  ;

: screenbase@   ( -- addr )
    9005  c@  0f0  and  6  lshift  9002  c@  80  and  2  lshift  or  vic2cpu  ;

: charbase@   ( -- addr )
    9005  c@ 0f  and  0a  lshift  vic2cpu  ;

: colourbase@   ( -- addr )
    9002  c@  80  and  2  lshift  9400  or  ;

: xymax@   ( -- maxx maxy )
    9002  c@  7f  and  9003  c@  7e  and  2/  ;

: xypos@   ( -- posx posy )
    9000  c@  7f  and  9001  c@  ;

: screenbase!   ( addr -- )
    cpu2vic  dup  6  rshift  0f0  and  9005  c@  0f  and  or  9005 c!
    2  rshift  80  and  9002  c@  7f  and  or  9002  c!  ;

: charbase!   ( addr -- )
    cpu2vic  0a  rshift  0f  and  9005  c@  0f0  and  or  9005  c!  ;

: xymax!   ( maxx maxy -- )
    2*  7e  and  9003  c@  81  and  or  9003  c!
    9002  c@  80  and  or  9002  c!  ;

: xypos!   ( posx posy -- )
    9001  c!  7f and  9000  c@  80  and  or  9000  c!  ;

( mode offsets )
( 0 - flags )
1 constant .screenbase
3 constant .charbase
5 constant .maxx
6 constant .maxy
7 constant .posx
8 constant .posy

: xycell   ( pixx pixy flags -- cellx celly )
    >r  r@  bitmap  and  if
	3  r@  double-height  and  if  1+  then  rshift
	swap  3  r>  multi-colour  and  if  1-  then  rshift
    swap  else  r>  drop  then  ;

: centre-screen   ( maxx maxy flags -- posx posy )
    >r  >r  ?ntsc  if  1b  else  22  then  swap  -
    ?ntsc  if  47  else  54  then  r>  2*  r>  double-height  and  if  2*  then  -  ;

: mode   ( maxx maxy flags -- )
    2  pick  2  pick  2  pick  create
    c,  screenbase@  ,  charbase@  ,  swap  c,  c,
    dup  >r  xycell  r>  centre-screen  swap  c,  c,  ;

variable current-mode

: xycellmax   ( mode -- cellx celly )
    dup  .maxx  +  c@  over  .maxy  +  c@  rot  c@  xycell  ;

: mode-set   ( mode -- )
    >r  r@  current-mode  !
    r@  .screenbase  +  @  screenbase!  r@  .charbase  +  @  charbase!
    r@  xycellmax  xymax!  r@  .posx  +  c@  r@  .posy  +  c@  xypos!
    r>  c@  double-height  and  9003  c@  0fe  and  or  9003  c!  ;

: colourbase   ( mode -- addr )
    .screenbase  +  @  200  and  9400  or  ;

: tile-screen   ( -- )
    current-mode  @  dup  .screenbase  +  @  swap  xycellmax  0
    2  pick  0  do
	over  0  do
	    dup  3  pick  i  *  j  +  5  pick  +  c!  1+
	loop
    loop  2drop  2drop  ;

: fill-colour   ( -- )
    current-mode  @  dup  colourbase  swap  dup  xycellmax  *  swap
    c@  multi-colour  and  286  c@  or  fill  ;

: zero-bmap   ( -- )
    current-mode  @  dup  .charbase  +  @  swap  dup  xycellmax  *
    swap  c@  8  swap  double-height  and  if  2*  then  *  erase  ;

: blank-screen   ( -- )
    current-mode  @  dup  .screenbase  +  @  swap  xycellmax  *  blank  ;

: clear-screen   ( -- )
    current-mode  @  c@  bitmap  and  if
	zero-bmap  tile-screen
	else  blank-screen
    then  fill-colour  ;

: char-at   ( x y -- addr )
    current-mode  @  .maxx  +  c@  *  +
    current-mode  @  .screenbase  +  @  +  ;

: colour-at   ( x y -- addr )
    current-mode  @  >r  r@  c@  xycell  r@  .maxx  +  c@  0  r@  c@
    xycell  drop  *  +  r>  colourbase  +  ;

: byteof   ( x y -- offset )
    swap  3  current-mode  @  c@  multi-colour  and
    if  1-  then  rshift  current-mode  @  .maxy  +  c@  *  +  ;

: maskfor   ( v x -- update mask )
    current-mode  @  c@  multi-colour  and  if
	3  and  3  swap  -  2*  >r  3  and  r@  lshift  3  else
	7  and  7  swap  -  >r  1  and  r@  lshift  1  then
    r>  lshift  not  ;

: plot   ( n x y -- )
    over  swap  byteof  current-mode  @  .charbase  +  @  +  >r  r@  c@
    >r  maskfor  r>  and  or  r>  c!  ;

( Character set addresses )
8000 constant charset1
8800 constant charset2

: setchar   ( addr start count -- )
    3  current-mode  @  c@  double-height  and  if  1+  then
    >r  r@  lshift  swap  r>  lshift  current-mode  @  .charbase  +
    @  +  swap  cmove  ;


: reset-screen
    0e5c3  sys  blue  foreground  ;

' reset-screen  ex-error  catch

forth definitions
decimal
